@@ -340,7 +340,7 @@ class AppMenuButton {
         this.window_signals.connect(this.metaWindow, 'notify::title', this.setDisplayTitle, this);
         this.window_signals.connect(this.metaWindow, "notify::minimized", this.setDisplayTitle, this);
         this.window_signals.connect(this.metaWindow, "notify::tile-type", this.setDisplayTitle, this);
-        this.window_signals.connect(this.metaWindow, "notify::icon", this.setIcon, this);
+        this.window_signals.connect(this.metaWindow, "icon-changed", this.setIcon, this);
         this.window_signals.connect(this.metaWindow, "notify::appears-focused", this.onFocus, this);
         this.window_signals.connect(this.metaWindow, "unmanaged", this.onUnmanaged, this);
     }
@@ -993,7 +993,7 @@ class CinnamonWindowListApplet extends Applet.Applet {
         this.settings.bind("window-preview-show-label", "showLabel", this._onPreviewChanged);
         this.settings.bind("window-preview-scale", "previewScale", this._onPreviewChanged);
 
-        this.signals.connect(global.screen, 'window-added', this._onWindowAddedAsync, this);
+        this.signals.connect(global.screen, 'window-added', this._onWindowAdded, this);
         this.signals.connect(global.screen, 'window-monitor-changed', this._onWindowMonitorChanged, this);
         this.signals.connect(global.screen, 'window-workspace-changed', this._onWindowWorkspaceChanged, this);
         this.signals.connect(global.screen, 'window-skip-taskbar-changed', this._onWindowSkipTaskbarChanged, this);
@@ -1090,16 +1090,14 @@ class CinnamonWindowListApplet extends Applet.Applet {
         this.manager.set_spacing(spacing * global.ui_scale);
     }
 
-    _onWindowAddedAsync(screen, metaWindow, monitor) {
-        Mainloop.timeout_add(20, Lang.bind(this, this._onWindowAdded, screen, metaWindow, monitor));
-    }
-
     _onWindowAdded(screen, metaWindow, monitor) {
         if (this._shouldAdd(metaWindow))
             this._addWindow(metaWindow, false);
     }
 
     _onWindowMonitorChanged(screen, metaWindow, monitor) {
+        const windowActor = metaWindow.get_compositor_private();
+        if (!windowActor || (!windowActor.visible && !metaWindow.minimized)) return;
         if (this._shouldAdd(metaWindow))
             this._addWindow(metaWindow, false);
         else
@@ -1114,9 +1112,7 @@ class CinnamonWindowListApplet extends Applet.Applet {
     }
 
     _onWindowSkipTaskbarChanged(screen, metaWindow) {
-        let window = this._windows.find(win => (win.metaWindow == metaWindow));
-
-        if (window && window.is_skip_taskbar()) {
+        if (metaWindow && metaWindow.is_skip_taskbar()) {
             this._removeWindow(metaWindow);
             return;
         }
